// Etape 1 :
// Importation des packages Java 2
import java.applet.Applet;
import java.awt.*;

// Etape 2 :
// Importation des packages Java 3D
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import javax.media.j3d.*;

public class Morphing extends Applet {

  public Morphing() {
    this.setLayout(new BorderLayout());

    // Etape 3 :
    // Creation du Canvas 3D
    Canvas3D canvas3D =
        new Canvas3D(SimpleUniverse.getPreferredConfiguration());

    this.add(canvas3D, BorderLayout.CENTER);

    // Etape 4 :
    // Creation d'un objet SimpleUniverse
    SimpleUniverse simpleU = new SimpleUniverse(canvas3D);

    // Etape 5 :
    // Positionnement du point d'observation pour avoir une vue correcte de la
    // scene 3D
    simpleU.getViewingPlatform().setNominalViewingTransform();

    // Etape 6 :
    // Creation de la scene 3D qui contient tous les objets 3D
    // que l'on veut visualiser
    BranchGroup scene = createSceneGraph();

    // Etape 7 :
    // Compilation de la scene 3D
    scene.compile();

    // Etape 8:
    // Attachement de la scene 3D a l'objet SimpleUniverse
    simpleU.addBranchGraph(scene);
  }

  /**
   * Creation de la scene 3D qui contient tous les objets 3D
   * @return scene 3D
   */
  public BranchGroup createSceneGraph() {

    // Creation de l'objet parent qui contiendra tous les autres objets 3D
    BranchGroup parent = new BranchGroup();

    // Creation de l'animation temporelle
    Alpha alpha = new Alpha(-1, 4000);
    alpha.setMode(Alpha.INCREASING_ENABLE | Alpha.DECREASING_ENABLE);
    alpha.setDecreasingAlphaDuration(2000);
    alpha.setStartTime(System.currentTimeMillis());

    // Construction du cube
    ColorCube cube =  new ColorCube(0.2f);

    // Construction de la pyramide
    Pyramide pyramide = new Pyramide();

    // Recuperation de la geometrie de ces deux objets
    GeometryArray[] geometries = new GeometryArray[2];
    geometries[0] = (GeometryArray)cube.getGeometry();
    geometries[1] = (GeometryArray)pyramide.getGeometry();

    // Creation du morphing qui va transformer le cube en pyramide
    Morph morphing = new Morph((GeometryArray[])geometries);
    morphing.setCapability(Morph.ALLOW_WEIGHTS_READ);
    morphing.setCapability(Morph.ALLOW_WEIGHTS_WRITE);

    // Creation du comportement associe au morphing
    MorphingBehavior morphBehavior = new MorphingBehavior(alpha, morphing);
    morphBehavior.setSchedulingBounds(new BoundingSphere());

    // Creation d'une transformation (rotation X suivie de rotation Y) que
    // l'on va appliquer a l'objet morphing
    TransformGroup transformG = new TransformGroup();
    Transform3D rotationX = new Transform3D();
    rotationX.rotX(30 * Math.PI / 180);
    Transform3D rotationXY = new Transform3D();
    rotationXY.rotY(30 * Math.PI / 180);
    rotationXY.mul(rotationX);
    transformG.setTransform(rotationXY);

    // Creation du groupe de transformation sur lequel va s'appliquer l'action
    // de la souris
    TransformGroup mouseTransform = new TransformGroup();

    // Le groupe de transformation sera modifie par le comportement de la
    // souris
    mouseTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    mouseTransform.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

    // Creation comportement rotation a la souris (sensibilite 0.05)
    MouseRotate rotate = new MouseRotate(mouseTransform);
    rotate.setSchedulingBounds(new BoundingSphere());

    // Ajout de comportement de la souris dans le graphe de la scene
    mouseTransform.addChild(transformG);
    parent.addChild(mouseTransform);
    parent.addChild(rotate);

    // Ajout du morphing au groupe de transformation
    transformG.addChild(morphing);

    // Ajout du comportement associe au morphing dans le graphe de la scene
    transformG.addChild(morphBehavior);

    return parent;
  }

  /**
   * Etape 9 :
   * Methode main() nous permettant d'utiliser cette classe comme une applet
   * ou une application.
   * @param args arguments de la ligne de commande
   */
  public static void main(String[] args) {
    Frame frame = new MainFrame(
      new Morphing(), 256, 256);
  }
}
