import java.util.*;
import javax.media.j3d.*;

/**
 * Creation d'un comportement personnalise pour le morphing
 */
public class MorphingBehavior extends Behavior {

  private Alpha alpha = null;
  private Morph morph = null;
  private double weights[] = null;

  private WakeupOnElapsedFrames wakeupCondition = null;

  /**
   * Construit le comportement lie au morphing entre le cube et la pyramide
   * @param alpha parametre temporel de l'animation
   * @param morph morphing entre le cube et la pyramide
   */
  public MorphingBehavior(Alpha alpha, Morph morph) {
    this.alpha = alpha;
    this.morph = morph;
    weights = morph.getWeights();
    wakeupCondition = new WakeupOnElapsedFrames(0);
  }

  /**
   * Initialisation du comportement :
   * La methode processStimulus() sera appelee a chaque fois qu'une frame
   * sera rendue
   */
  public void initialize() {
    alpha.setStartTime(System.currentTimeMillis());
    wakeupOn(wakeupCondition);
  }

  /**
   * Methode appelee a chaque fois qu'une frame est affichee
   * @param criteria critere ayant declenche le stimulus
   */
    public void processStimulus(Enumeration criteria) {
      WakeupCriterion critere;
      double alphaValue = alpha.value();

      // On boucle sur les criteres ayant declenche le comportement
      while (criteria.hasMoreElements()) {
        // On recupere le premier critere de l'enumeration
        critere = (WakeupCriterion)criteria.nextElement();

        // On ne traite que les criteres correspondant a WakeupOnElapsedFrames
        if (critere instanceof WakeupOnElapsedFrames) {
          weights[0] = 1.0 - alphaValue;
          weights[1] = alphaValue;
          morph.setWeights(weights);
        }
      }  // fin while (criteria.hasMoreElements())

      // Une fois le stimulus traite, on reinitialise le comportement
      this.wakeupOn(wakeupCondition);
    }
}
